/*
 * Created on Dec 19, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.List;

import com.mckessonaps.jspwiki.plugin.query.operators.Modifier;
import com.mckessonaps.jspwiki.plugin.query.operators.Operator;

/**
 * Expressions can be nested, combined with Operators and contain Terms. An expression when evaluated against
 * a canidate page, returns true if the canidate page is acceptable it it and all it's nested parts.
 * 
 * @author jvolkar
 */
public class Expression implements Operand
{
    /**
	 * This parser instance is used to do the string handling, it checks the string and builds up a list of
	 * logical elements that Expression attempts to disambiguate.
	 */
    private static ExpressionParser m_parser = new ExpressionParser();

    //Every expresion has a left + operator + right
    private Operand m_left;
    private Operator m_operator;
    private Operand m_right;
    

    /**
	 * Main parsing logic to turn a string into an expression is all in ExpressionParser. Yuck, someday I
	 * really need to learn ANTLR or JavaCC...
	 * 
	 * @param queryString
	 * @return
	 */
    public static Expression parse(String queryString)
    {
        return m_parser.parse(queryString);
    }

    /**
	 * Constructor, takes a list of Terms, Operators and sub-Expressions that make up this expression.
	 * formalizes them into a proper series of nesting expressions, result is one top-level expression and a
	 * structured tree inside of it.
	 * 
	 * @param elements
	 */
    Expression(List elementList)
    {
        //Start off stupid, (true AND true) is true...
        m_left = Operand.NULL;
        m_operator = Operator.NULL;
        m_right = Operand.NULL;

        if (null == elementList) //Ignore null lists...
            return;

        if (elementList.size() == 0) //ignore empty lists...
            return;

        processElements(elementList);

    } //constructor

    /**
	 * Ick ack ook, I really gotta learn to write real lexers/parsers, this verges on the embarassing....
	 * (Hmm, verges hell, REFACTOR this! Hrummp, break statements!)
	 * 
	 * @param elementList
	 */
    private void processElements(List elementList)
    {
        for (int i = 0; i < elementList.size(); i++)
        {
            Object element = elementList.get(i);

            if (isOperator(element))
            {
                Operator o = (Operator) element;

                if (o.isInfix())
                {
                    if (haveOperator())
                        throw new RuntimeException("Consequtive infix operators cannot be interperated.");

                    m_operator = o;
                }
                else //o.isPrefix()
                {
                    if ((haveLeft()) && (!haveOperator()))
                    {    
                        m_operator = Operator.AND; //provide an AND instead, don't increment i...
                        m_right = newExpressionFromRemaining(elementList, i);
                        break;
                    }
                    
                    if (haveOperator())
                    {
                        m_right = newExpressionFromRemaining(elementList, i);
                        break;
                    }
                    
                    m_operator = o;
                    m_right = newExpressionFromRemaining(elementList, i + 1);
                    break;
                    
                } //if infix/prefix
                
            } //is operator
            else if (isOperand(element))
            {
                if (noLeftAndNoOperator())
                {    
                    m_left = (Operand) element;
                }
                else
                {
                    m_right = newExpressionFromRemaining(elementList, i);
                    break;
                }

            } //is operand
            else
            {
                throw new RuntimeException("Not an operator, not an operand, what is it?");
            }

        } //for
    }

    
    private Expression newExpressionFromRemaining(List elementList, int i)
    {
        List subList = elementList.subList(i, elementList.size());

        if ((subList.size() == 1) && (subList.get(0) instanceof Expression))
            return (Expression) subList.get(0);

        return new Expression(subList);
    }

    private boolean haveLeft()
    {
        return (Operand.NULL != m_left);
    }

    private boolean haveOperator()
    {
        return (Operator.NULL != m_operator);
    }

    /**
	 * @return True if we still have no left operand and not explict operator.
	 */
    private boolean noLeftAndNoOperator()
    {
        return (!haveLeft() && !haveOperator());
    }

    /**
	 * @param element
	 * @return True if element is a Operand (Term or Expression)
	 */
    private boolean isOperand(Object element)
    {
        return (Operand.class.isAssignableFrom(element.getClass()));
    }

    /**
	 * @param element
	 * @return True if element is a Operator
	 */
    private boolean isOperator(Object element)
    {
        return (Operator.class.isAssignableFrom(element.getClass()));
    }


    /**
	 * Returns formal form of this expression.
	 */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        String left = m_left.toString();
        String operator = m_operator.toString();
        String right = m_right.toString();

        buffer.append("(");

        buffer.append(left);

        //ick...
        if ((!left.equals("")) && ((!operator.equals("")) || (!right.equals(""))))
            buffer.append(" ");

        buffer.append(operator);

        if (!operator.equals(""))
            buffer.append(" ");

        buffer.append(right);

        buffer.append(")");

        return buffer.toString().trim();
    }

    
    /**
     * Typically this method is only called by the Query itsself to ask if the 
     * top-level expression would accept the pageName as being part of the 
     * result set.
     *  
     * @param pageName
     * @param refInfoProvider
     * @return True if pageName should be in the result set.
     */
    public boolean accepts(String pageName, ReferenceInfoProvider refInfoProvider)
    {
        return evaluate(null, refInfoProvider, pageName);
    }
    
    
    /**
	 * @see Operand
	 */
    public boolean evaluate(Modifier modifier, ReferenceInfoProvider refInfoProvider, String pageName)
    {
        return m_operator.evaluate(m_left, m_right, modifier, refInfoProvider, pageName);
    }

}
