/*
 * Created on Dec 22, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import com.mckessonaps.jspwiki.plugin.query.operators.Operator;

/**
 * This is the first line of defense against bad query strings.  Parses from the 
 * original string form into Terms, Operators and if the user explictly had 
 * parenthesis, sub-Expressions.
 * 
 * @author jvolkar
 */
class ExpressionParser
{
    /**
     * Parse the string and return one top-level expression which if toString'd would show 
     * the cannonical form that the user should have given if his syntax was perfect.
     * @param queryString
     * @return
     */
    Expression parse(String queryString)
    {
        if ((null == queryString) || queryString.trim().equals(""))
            return new Expression( null );
            
        syntaxCheck(queryString);
        
        //at ths point we know that []'s and ()'s are reasonable so we can try...
        
        
        //queryString gets split on ()[] and space, return list is long and ugly...
        List list = chunk(queryString);
       
        //[]'s are used to collapse sublists into Terms, return list has ()'s and text and Terms
        list = collapseBracketedTerms(list);
       
        //Keywords are replaced with Operator instances...
        list = replaceOperators(list);
       
        //()'s are used to create Expression objects, return list structure is 
        //indeterminate at this point, greratly depended on users original queryString.
        //The list at this point could be idiotic: "AND, AND, OR, TermA, TermB" and
        //such is possible if users fed us junk. 
        list = explictExpressions(list);

        //If the user was VERY explict we might just have one expression...
        if ((list.size() == 1) && (list.get( 0 ) instanceof Expression))
            return (Expression)list.get( 0 );
            
            
        //It's up to Expression of make sense of the list...
        return new Expression(list);        
        
    }

    
    
    /**
     * Bust up the string into chunks for later processing...
     */
    private List chunk(String s)
    {
        ArrayList list = new ArrayList();
       
        StringTokenizer tokenizer = new StringTokenizer(s, "()[] ", true);
       
        while ( tokenizer.hasMoreTokens() )
            list.add( tokenizer.nextToken() ); 
       
        return list;
    }
   
    /**
     * Join back together the bracketed page names...
     * @param inList
     * @return
     */
    private List collapseBracketedTerms(List inList)
    {
        List outList = new ArrayList(inList.size() );
        StringBuffer termBuffer = new StringBuffer();
        boolean inTerm = false;
       
         for (Iterator iter = inList.iterator(); iter.hasNext();)
         {
             String chunk = (String) iter.next();
            
             if (chunk.equals( "[" ))
             { 
                 termBuffer.setLength( 0 );
                 inTerm = true;                
             }
            
             if (inTerm)
                 termBuffer.append( chunk );
             else
                 outList.add( chunk );
                
             if (chunk.equals( "]" ))
             {
                 inTerm = false;
                 outList.add( new Term( termBuffer.toString()) );
             }            
         } 
       
        return outList;
    }

    /**
     * Replace string names of operators with Operator instances.
     * If there is unrecognizable text throw, returned list will only have "(", ")",Term and
     * Operator instances in it.
     * @param inList
     * @return
     */
    private List replaceOperators(List inList)
    {
        List outList = new ArrayList(inList.size() );
       
        for (Iterator iter = inList.iterator(); iter.hasNext();)
        {
            Object nextObject = iter.next();
           
            //Just add Terms into the outList without modification...
            if (!(nextObject instanceof String))
            {    
                outList.add(nextObject);
                continue;
            }
           
            //Inspect the string....
            String chunk = (String) nextObject;
           
            //Replace operators, drop spaces, preserve parenthesis, anyhting else is an error...
            if (chunk.equalsIgnoreCase( "AND" ))
                outList.add( Operator.AND );
            else if (chunk.equalsIgnoreCase( "OR" ))
                outList.add( Operator.OR );
            else if (chunk.equalsIgnoreCase( "NOT" ))
                outList.add( Operator.NOT );
            else if (chunk.equalsIgnoreCase( "IS" ))
                outList.add( Operator.IS );
            else if (chunk.equalsIgnoreCase( "TO" ))
                outList.add( Operator.TO );
            else if (chunk.equalsIgnoreCase( "BY" ))
                outList.add( Operator.BY );
            else if (chunk.equals( " " ))
                 ;//drop space chunks...
            else if (chunk.equals( "(" ))               
                outList.add(chunk);           
            else if (chunk.equals( ")" ))               
                outList.add(chunk);
            else
                throw new RuntimeException("Unknown operator in page='' expression.");
        }
       
        return outList;
    }

    
    /**
     * Create expressions based on users use of expicit parenthesis...
     * @param inList
     * @return
     */ 
    private List explictExpressions(List inList)
    {
        //loop over inList, find last open before close, from that open to close 
        //is a sub-expression, recurse...
        int lastOpen = findLastOpen(inList);
        if (lastOpen == -1)
            return inList;
        
        int matchingClose = findNextClose(inList, lastOpen);
        if (matchingClose == -1)
            throw new RuntimeException("Mismatched or unbalanced ()'s in page=''.");

        
        List tempList = inList.subList( lastOpen + 1, matchingClose);
        
        Expression subExpression;
        if ((tempList.size() == 1) && (tempList.get(0) instanceof Expression))
            subExpression = (Expression) tempList.get(0);
        else
            subExpression = new Expression( tempList );
        
        tempList = new ArrayList();
        tempList.addAll( inList.subList( 0, lastOpen) );
        tempList.add(subExpression);
        tempList.addAll( inList.subList( matchingClose + 1, inList.size() ) );
        
        return explictExpressions( tempList );
    }

    
    private int findLastOpen(List inList)
    {
        int lastOpen = -1; //not found...
        
        for(int i =0; i < inList.size(); i++)
        {
            Object element = inList.get( i );
            
            if (! (element instanceof String))
                continue;
            
            String s = (String)element;
            
            if (s.equals( "(" ))
                lastOpen = i;
            
            if ( s.equals(")") )
                break;
        }   
        
        return lastOpen;
    }
    
    
    private int findNextClose(List inList, int startIndex)
    {
        int nextClose = -1; //not found...
        
        for(int i = startIndex; i < inList.size(); i++)
        {
            Object element = inList.get( i );
            
            if (! (element instanceof String))
                continue;
            
            String s = (String)element;
            
            if (s.equals( ")" ))
            {    
                nextClose = i;
                break;
            }
        }   
        
        return nextClose;
    }
   
    /**
     * Do some basic sanity checking on the string to be sure we can parse it.
     * @param s
     */
    private void syntaxCheck(String s)
    {
        char[] chars = s.toCharArray();
       
        syntaxCheckBrackets(chars);
       
        syntaxCheckParenthesis(chars);
    }
   
   
    /**
     * Square brackets must exist balance and NOT nest...
     * @param chars
     */
    private void syntaxCheckBrackets(char[] chars )
    {
        int leftCount = 0;
        int rightCount = 0;
       
        for (int i = 0; i < chars.length; i++)
        {    
            if ('[' == chars[i])
                leftCount++;
           
            if (']' == chars[i])
                rightCount++;

            if ((leftCount - rightCount) > 1)
                throw new RuntimeException("[]'s are nested.");

            if (rightCount > leftCount)
                throw new RuntimeException("[]'s are mismatched.");
        }
       
        if (rightCount != leftCount)
            throw new RuntimeException("[]'s are not balanced.");
           
        if ((rightCount == 0) && (leftCount == 0 ))
            throw new RuntimeException("Page names must be surrounded by []'s.");
    }
   
    /**
     * Parenthesis must balance and nest correctly.
     * @param chars
     */
    private void syntaxCheckParenthesis(char[] chars )
    {
        int leftCount = 0;
        int rightCount = 0;
       
        for (int i = 0; i < chars.length; i++)
        {    
            if ('(' == chars[i])
                leftCount++;
           
            if (')' == chars[i])
                rightCount++;

            if (rightCount > leftCount)
                throw new RuntimeException("()'s are mismatched.");
        }

        if (rightCount != leftCount)
            throw new RuntimeException("()'s are not balanced.");
       
    }
    
    
}
