/*
 * Created on Dec 15, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import com.ecyrd.jspwiki.ReferenceManagerSpy;
import com.ecyrd.jspwiki.WikiEngine;

/**
 * @author jvolkar
 */
class Query
{
    private final String m_queryString;
    private final Expression m_expression;
    
    private long m_lastAccess = System.currentTimeMillis();
    private Set m_cachedResultSet;
    
    /**
     * Constructor, attempts to parse the queryString, if unparsable throws a runtime exception.
     * @param queryString
     */
    Query(String queryString)
    {
        m_queryString = queryString;
        m_expression = Expression.parse( queryString );
    }
    
    
    long getLastAccess()
    {
        return m_lastAccess;
    }
    
    String getQueryString()
    {
        return m_queryString;
    }
    
    String getExpressionString()
    {
        return m_expression.toString();
    }
    
    /**
     * 
     * @param engine
     * @return
     */
    Set getResults(WikiEngine engine)
    {        
        long now = System.currentTimeMillis();
        
        //NOTE: If there was someway to tell if the manager hadn't changed since we last
        //did this, we could cache the resultsfor an indeterminate amount of time...
        //if (ReferenceManagerSpy.isDirty()
        
        if ((now - m_lastAccess) > 30000) //last access more than 30 seconds ago...
            m_cachedResultSet = buildResultsSet(engine);
        
        if (null == m_cachedResultSet)
            m_cachedResultSet = buildResultsSet(engine);
        
        m_lastAccess =now;
        return m_cachedResultSet; 
    }
    
    /**
     * Evaluate our expression against the references maintained by the manager.
     * @return A list of the pages that are accepted by our expression, or an empty list oif no pages
     * match.  Will not return null.
     */
    private Set buildResultsSet(WikiEngine engine)
    {
        Set resultsSet;

        //Use the toMap rather than engine.getPageManager().getAllPages(); as that
        //actually walks the filesystem on the most commonly used provider (VersioningFileProvider)
        ReferenceManagerSpy spy = new ReferenceManagerSpy(engine);
        
        Map toMap = spy.getRefersTo();
        
        resultsSet = evaluateExpression(spy, toMap.keySet());
        return resultsSet; 
    }
    
    
    /**
     * @param pageSet Set of all pages to be evaluated.
     * @return Non null list.
     */
    private Set evaluateExpression(ReferenceInfoProvider refInfoProvider, Set pageSet)
    {
        Set resultSet = new TreeSet();

        for (Iterator pageNameIter = pageSet.iterator(); pageNameIter.hasNext();)
        {
            String pageName = (String) pageNameIter.next();
            
            if ( m_expression.accepts(pageName, refInfoProvider) )
                resultSet.add( pageName );
        }
        
        return resultSet;
    }
    
}
