/*
 * Created on Dec 15, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ecyrd.jspwiki.WikiEngine;

/**
 * Manages prebuilt Query objects, so the plugin doesn't have to re-parse and 
 * such on each display of a wikipage.  Handles case of possibly same queryString 
 * but on different WikiEngine instances (different wiki's).
 * 
 * @author jvolkar
 */
class QueryManager
{
    private Map m_engineMapOfQueryMaps = new HashMap();
    
    private List m_queryList = new ArrayList(); 
    
    QueryManager()
    {
        QueryReaper.startOnce();
        QueryReaper.register(this);
    }
    
    /**
     * @return a Copy of a list of all current queries, used internally by QueryReaper.
     */
    List getQueryList()
    {
        //To avoid potential synch issues with reaper return a copy...
        return new ArrayList( m_queryList );
    }
    
    /**
     * The SAME queryString, evaluated against DIFFERENT engines probably will not
     * give the same results (typically different engine instance means different wiki's.
     * So, we keep a map of engines, holding a map of queryStrings for each.
     * 
     * @param engine 
     * @param queryString
     * @return 
     */
    synchronized Query getQuery(WikiEngine engine, String queryString)
    {
        Map queryMap = (Map)m_engineMapOfQueryMaps.get( engine );
        if (null == queryMap)
        {
            queryMap = new HashMap();
            m_engineMapOfQueryMaps.put( engine, queryMap );
        }
        
        Query q = (Query) queryMap.get( queryString );
        if (null == q)
        {    
            q = new Query(queryString);
            queryMap.put( queryString, q ); //add to the map which is our "cache"...
            m_queryList.add( q ); //add to our (redundant) list of all queries for the reaper...
        }
        
        return q;
    }
    
    /**
     * Called by the QueryReaper when it finds that the query must die.
     * @param q
     */
    synchronized void remove(Query q)
    {
        m_queryList.remove( q );
        
        for (Iterator queryMapIter = m_engineMapOfQueryMaps.values().iterator(); queryMapIter.hasNext();)
        {
            Map queryMap = (Map) queryMapIter.next();
            
            if (queryMap.containsValue( q ))
               queryMap.remove( q.getQueryString() );
            
        }
    }
    
}
