/*
 * Created on Dec 15, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.WikiEngine;
import com.ecyrd.jspwiki.plugin.WikiPlugin;

/**
 * @author jvolkar
 */
public class QueryPlugin implements WikiPlugin
{
    private static QueryManager c_queryManager = new QueryManager(); 


    /* (non-Javadoc)
     * @see com.ecyrd.jspwiki.plugin.WikiPlugin#execute(com.ecyrd.jspwiki.WikiContext, java.util.Map)
     */
    public String execute(WikiContext context, Map params) 
    {
        return execute(context.getEngine(), params);
    }

    
    /**
     * Real working body of standard {@link WikiPlugin#execute(com.ecyrd.jspwiki.WikiContext, java.util.Map)}
     * Separated out for testability.
     * 
     * @param manager
     * @param params
     * @return
     */
    String execute(WikiEngine engine, Map params)
    {
        String pageExpression = (String)params.get("pages");
        
        if ( null == pageExpression )
            return "Missing Parameter: 'pages'";
        
        Query query;
        try
        {
            query = c_queryManager.getQuery(engine, pageExpression);
        }
        catch (RuntimeException e)
        {
            return e.getMessage();
        }
        
        //For testing purposes emit the parsed form of the expression back out...
        params.put( "pages", query.getExpressionString() );
        
        Set resultSet = null;
        try
        {
            resultSet = query.getResults( engine );
        }
        catch (Exception e)
        {
            return "Error: " + e + " Query string parsed to: " + query.getExpressionString();
        }
        
        return processResultSet(engine, resultSet);
    }
    
    
    /**
     * Format the results of the query into an HTML output suitable for display on the page. 
     * @param resultsList - Must not be null.
     * @return XHTML compliant HTML fragment, all tags to be closed and balanaced.
     */
    protected String processResultSet(WikiEngine engine, Set resultSet)
    {
        if ( resultSet.isEmpty() )
            return "No pages found.";
        
        StringBuffer resultsBuffer = new StringBuffer();
        resultsBuffer.append( "<ul>" );
        for (Iterator iter = resultSet.iterator(); iter.hasNext();)
        {    
            String pageName = (String) iter.next();
            resultsBuffer.append( "<li><a href=\"" + engine.getViewURL( pageName ) + "\">" );
            resultsBuffer.append( pageName );
            resultsBuffer.append( "</a></li>" );
        }
        resultsBuffer.append( "</ul>" );
        
        return resultsBuffer.toString();
    }
    
}
