/*
 * Created on Dec 22, 2003
 */
package com.mckessonaps.jspwiki.plugin.query;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * Tells the QueryManager to remove Queries that haven't been accessed in some time.
 * 
 * (Is this really needed?  What if we just let them lay around, they consume memory, 
 * but surely not that much?  --JohnVolkar)  
 * 
 * @author jvolkar
 */
class QueryReaper extends Thread
{
    private static long FIVE_MINUTES = (5 * 60 * 1000);
    private static long ONE_HOUR = (1 * 60 * 60 * 1000);
    
    private static boolean c_reaperNeedsStarted = false;
    private static List c_managerList = new ArrayList();
    
    /**
     * Utility to allow QueryManager to assure that one and only one reaper gets started.
     */
    static synchronized void startOnce()
    {
        if (c_reaperNeedsStarted)
        {
            QueryReaper reaper = new QueryReaper();
            reaper.start();
            c_reaperNeedsStarted = false;
        }
    }
    
    static void register(QueryManager manager)
    {
        synchronized(c_managerList) //no new managers while reaping...
        {
            c_managerList.add(manager);
        }
    }
    
    /**
     * Hide so only can be started by startOnce()...
     */
    private QueryReaper()
    {
        setDaemon( true );
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run()
    {
        while(true)
        {
            idleABit();
            reapOldQueries();
        }
    }

    
    
    private void idleABit()
    {
        try
        {
            Thread.sleep( FIVE_MINUTES );
        }
        catch (InterruptedException e)
        {
            //Ignore...
        }
    }

    
    private void reapOldQueries()
    {
        synchronized(c_managerList) //no new managers while reaping...
        {
            for (Iterator managerIter = c_managerList.iterator(); managerIter.hasNext();)
            {
                QueryManager manager = (QueryManager) managerIter.next();
                
                List queryList = manager.getQueryList();
                
                reapOldQueries(manager, queryList);
            }
        }            
    }
    
    private void reapOldQueries(QueryManager manager, List queryList)
    {
        long now = System.currentTimeMillis();
        
        for (Iterator queryIter = queryList.iterator(); queryIter.hasNext();)
        {
            Query query = (Query) queryIter.next();
            
            if (isOld(query, now))
                manager.remove( query );
        }
    }
    
    private boolean isOld(Query q, long now)
    {
        //NOTE: This ignores rollover, but ignoring it only results in premature reaping so 
        //ignore it for now.
        return ( q.getLastAccess() < (now - ONE_HOUR));
    }
    
    
}
