package com.mckessonaps.jspwiki.plugin.ticker;


/**
 * Quote is a simple data holder for use in the caching mechanism.
 */
class Quote
{
    private static final long EXPIRATION_DELTA = 60000; //ms = 60sec = 1min
    
    private final String m_symbol;
    
    private String m_value;
    
    private long m_timestamp;
    
    
    /**
     * Basic constructor...
     * @param symbol
     * @param value
     */
    Quote(String symbol, String value)
    {
        m_symbol= symbol.trim();
        m_value = value.trim();
        m_timestamp = System.currentTimeMillis();
    }
    
    
    /**
     * Getter...
     * @return String
     */
    String getSymbol()
    {
        return m_symbol;
    }
    
    
    /**
     * Getter...
     * @return String
     */
    String getValue()
    {
        return m_value;
    }
    
    
    /**
     * @return boolean
     */
    boolean isExpired()
    {
        long expire = m_timestamp + EXPIRATION_DELTA;
        long current = System.currentTimeMillis();
        
        if (current < m_timestamp)
            return true; //rolled over...
        else
            return (expire > current);
    }


    /**
     * Update the value and freshen the expiration time stamp.
     * @param newValue
     */
    void update(String newValue)
    {
        m_value = newValue.trim();
        m_timestamp = System.currentTimeMillis();
    }
    
}
