/*
 * Created on Jan 21, 2004
 */
package com.mckessonaps.jspwiki.plugin.transclude;

import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClientLite;

import com.mckessonaps.jspwiki.plugin.util.WikiTextUtils;


/**
 * Utility class, keeps load off server by limiting real page fetches via XML-RPC 
 * when folks are viewing pages containing the TranscludePlugin. 
 * @author jvolkar
 */
class CachedPage
{
    private static final String SECTION_DIVIDER = "<hr />";
    
    private String m_html;
    private long m_lastUpdate;
    private final String m_rpcUrl;
    private final String m_name;
    private int m_cacheLife;
    private int m_sectionNumber;
    
    private final String m_divStyle = ""; //todo: extension parameter
    private final String m_divClass = "transclusion"; //todo: extension parameter
    
    
    CachedPage(String rpcUrl, String name, int cacheLife, int sectionNumber)
    {
        m_rpcUrl = rpcUrl;
        m_name = name;
        update(cacheLife, sectionNumber);
    }
    
    void update(int cacheLife, int sectionNumber)
    {
        m_cacheLife = cacheLife;
        m_sectionNumber = sectionNumber;
    }
    
    
    /**
     * @return Returns the html.
     */
    String getHtml()
    {
        long now = System.currentTimeMillis();
        long age = now - m_lastUpdate;
        
        if (age > m_cacheLife)
            refreshPage();
        
        return m_html;
    }

    /**
     * @return Returns the lastUpdate.
     */
    long getLastUpdate()
    {
        return m_lastUpdate;
    }

    /**
     * @return Returns the name.
     */
    String getName()
    {
        return m_name;
    }

    /**
     * Actually go out and get the darn text of the page via XML-RPC
     * @param context
     */
    private void refreshPage()
    {
        StringBuffer buffer = new StringBuffer(1024);
        buffer.append("<div style=\"" + m_divStyle + "\" class=\""+ m_divClass +"\">");
        try
        {
            XmlRpcClientLite client = new XmlRpcClientLite(m_rpcUrl);
            Vector args = new Vector();
            args.add( m_name );
            
            //todo: publish the WikiRpc class from the Visualizer and use it...
            //todo: This is likely a bogus way to deBase64 the return.
            String html = new String( (byte[])client.execute( "wiki.getPageHTML", args ), "UTF-8");
            
            if (m_sectionNumber > 0) 
                html = WikiTextUtils.extractSectionData( m_sectionNumber, html, SECTION_DIVIDER );
            
            buffer.append( html );
        }
        catch (Exception e)
        {
            buffer.append( "Error, failed to reach: " + m_rpcUrl + " Exception: " + e + " Message: " + e.getMessage() );
        } 
        buffer.append( "</div>");
        
        m_html = buffer.toString();        
    }
    
    
}