package com.mckessonaps.jspwiki.plugin.transclude;


import java.util.HashMap;
import java.util.Map;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.plugin.WikiPlugin;
import com.mckessonaps.jspwiki.plugin.util.ParamUtil;



/**
 * todo: TranscludePlugin uses a caching mechanism, we need some sort of reaper 
 * on it eventually.  Not abig problem as there are not that many transclusions.  
 * Also self-referring plugins would be BAD!
 * 
 * NOTE: This works best between JSPWiki based wiki's!  We retrieve the wiki-text 
 * for the foreign page, unless thier wiki markup matches what the JSPWiki's 
 * TranslatorReader expects you will have to set the translate parameter to false.
 * 
 * page = name of page, may be able to provide interwikilink syntax.
 * 
 * xmlrpc = url to foreign wiki's RPC interface.  This parameter is optional if 
 * you provide an interwiki link as the page parameter AND the RPC url is 
 * guessable from the interwiki link.  If the url cannot be guessed or the 
 * guess turns out tobe wrong, then this parameter is requried.
 * 
 * cache = a number of seconds for which a cache of the transcluded page is to 
 * be considered valid, default is 30.  This exists just to keep server loads in check.
 * 
 * translate = true or false, default is true.  Should we attempt to translate 
 * the retrieved wiki text into html or simple enclose it in a PRE section.  If 
 * false, the raw wiki text is preserved (a PRE tag surrounds the foriegn material 
 * and it's word wrapped at around80 charaters to keep from having to side-scroll).
 * 
 */
public class TranscludePlugin 
implements WikiPlugin
{
    
    private static Map c_cacheMap = new HashMap();
    

    /**
     * @see com.ecyrd.jspwiki.plugin.WikiPlugin#execute(com.ecyrd.jspwiki.WikiContext, java.util.Map)
     */
    public String execute(WikiContext context, Map paramMap) 
    {
        String specifiedPage = ParamUtil.getString( null, paramMap, "page", null);
        if (null == specifiedPage)
            return "You must provide a 'page' parameter.";

        String pageName = null;
        String interwiki = null;
        String xmlrpcUrl;
        int colonIndex = specifiedPage.indexOf( ":");
        if ( colonIndex > 0 ) //page contains a non leading ':'
        {
            pageName = specifiedPage.substring( colonIndex + 1 ); //chop off interwiki
            interwiki = specifiedPage.substring( 0, colonIndex ); //chop off page
        }
        
        if (null != interwiki)
            xmlrpcUrl = guessXmlRpcUrl(context, interwiki);
        else
            xmlrpcUrl = ParamUtil.getString( null, paramMap, "xmlrpc", null);

        if (null == xmlrpcUrl)
            return "You must provide the 'xmlrpc' parameter. (pageName=" + pageName +", interwiki=" + interwiki + ")";

        int cacheLife = ParamUtil.getInteger( null, paramMap, "cache", 30);
        int sectionNumber = ParamUtil.getInteger( null, paramMap, "section", -1);
        
        return transclude(xmlrpcUrl, pageName, cacheLife, sectionNumber);
    }
     
    
    /**
     * @return A guess at the Wiki RPC url based off a defined interwiki link.  
     * May return null if unable to guess.
     */
    private String guessXmlRpcUrl(WikiContext context, String interwiki)
    {
        String url = context.getEngine().getInterWikiURL( interwiki );
        
        if (null != url)
        {//This is a bogus way to guess the url, but works many setups...
            url = url.substring(0, url.lastIndexOf( "/") );
            url += "/RPC2/"; 
        }
        
        return url;
    }
    
    
    /**
     * This method is synchronized to protect the cache.  Could be done piecewise 
     * at a finer grain level, but this is easy for now.
     */
    private synchronized String transclude(String rpcUrl, String page, int cacheLife, int sectionNumber)
    {
        CachedPage cached = (CachedPage)c_cacheMap.get(rpcUrl + page);
        
        if (null == cached)
        {
            cached = new CachedPage(rpcUrl, page, cacheLife, sectionNumber);
        }
        else
        {
            //Always update these so plugin parames are fed down...
            cached.update( cacheLife, sectionNumber );
        }
        
        return cached.getHtml();
    }
    
}
