
package com.mckessonaps.jspwiki.plugin.util;

import java.util.Map;

import org.apache.log4j.Logger;

/**
 * Some simple parameter handling utilities.
 */
public class ParamUtil
{

    /**
     * Get the named parameter from the map, if it does not exist in the Map return the default 
     * value.  If the map value is not a string that can be parsed into an integer retun the 
     * default value.
     */
    public static int getInteger(Logger log, Map paramMap, String paramName, int paramDefault)
    {
        //If parameter is not specified use default....
        if (! paramMap.containsKey( paramName  ) )
        {
            if (null != log)
                log.debug( "Parameter '" + paramName + "' not provided, using default value: " + paramDefault );
            return paramDefault;
        }
            
        Object value = paramMap.get( paramName );
        if (!(value instanceof String))
        {
            if (null != log)
                log.warn( "Parameter '" + paramName + "' is not a String value, using default value: " + paramDefault );
            return paramDefault;
        }

        int intValue = paramDefault;
        try
        {
            intValue = Integer.parseInt( (String) value );
        }
        catch(NumberFormatException nfe)
        {
            if (null != log)
                log.warn("Parameter '" + paramName + "' is not a parsable to an integer value, using default value: " + paramDefault );
        }
        
        if (null != log)
            log.info("Parameter '" + paramName + "' now set to: " + intValue);
        return intValue;
    }



    /**
     * Get the named parameter from the map, if it does not exist in the Map return the default 
     * value.  Assures the the returned string is trimmed (no leading or trailing spaces).
     */
    public static String getString(Logger log, Map paramMap, String paramName, String paramDefault)
    {
        //If parameter is not specified use default....
        if (! paramMap.containsKey( paramName  ) )
        {
            if (null != log)
                log.debug( "Parameter '" + paramName + "' not provided, using default value: " + paramDefault );
            return paramDefault;
        }
            
        Object value = paramMap.get( paramName );
        if (!(value instanceof String))
        {
            if (null != log)
                log.warn( "Parameter '" + paramName + "' is not a String value, using default value: " + paramDefault );
            return paramDefault;
        }

        String stringValue = (String)value;
        stringValue = stringValue.trim();

        if (null != log)
            log.info("Parameter '" + paramName + "' now set to: " + stringValue);
        
        return stringValue;
    }

    
    
    /**
     * Get the named parameter from the map, if it does not exist in the Map return the default 
     * value.  If the map value cannot be infered as an affirmitive then false is returned.
     * 
     * Infers the followin as affiritive: "yes", "true", and "on"
     */
    public static boolean getBoolean(Logger log, Map paramMap, String paramName, boolean paramDefault)
    {
        //If parameter is not specified use default....
        if (! paramMap.containsKey( paramName  ) )
        {
            if (null != log)
                log.debug( "Parameter '" + paramName + "' not provided, using default value: " + paramDefault );
            return paramDefault;
        }
            
        Object value = paramMap.get( paramName );
        if (!(value instanceof String))
        {
            if (null != log)
                log.warn( "Parameter '" + paramName + "' is not a String value, using default value: " + paramDefault );
            return paramDefault;
        }

        String stringValue = (String)value;
        stringValue = stringValue.trim();
        
        boolean boolValue = false;
        boolValue &= stringValue.equalsIgnoreCase("true");
        boolValue &= stringValue.equalsIgnoreCase("on");
        boolValue &= stringValue.equalsIgnoreCase("yes");
        
        if (null != log)
            log.info("Parameter '" + paramName + "' now set to: " + boolValue);
        
        return boolValue;
    }
    
    

}
