
package com.mckessonaps.jspwiki.plugin.util;

import java.util.Map;

/**
 * 
 */
public abstract class Parameter
{
    private final String m_name;
    private final String m_description;
    private final boolean m_required;
    private boolean m_usingDefault = true;
    private String m_suppliedValue;
    
    public Parameter(String name, boolean required, String description, ParameterGroup group) 
    {
        m_name = name;
        m_required = required;
        m_description = description;
        group.add( this );
    }
    
    public String getSuppliedValue()
    {
        return m_suppliedValue;
    }
    
    public void throwBad(String extraMessage) throws ParameterException
    {
        String message = "Bad value '" + getSuppliedValue() + "' for parameter '" + getName() + "'.  ";
        
        throw new ParameterException(message + extraMessage);
        
    }
    
    public void update(Map stringParamMap) throws ParameterException
    {
        String stringValue = (String)stringParamMap.get( getName() );
        
        if (null == stringValue && isRequired())    
            throw new ParameterException( this );
        
        if (null == stringValue)
        {    
            m_suppliedValue = "";
            return;
        }
            
        m_suppliedValue = stringValue.trim();
        parseSuppliedValue( );
        m_usingDefault = false;
        
    }
    
    protected abstract void parseSuppliedValue()
    throws ParameterException;
    
    /**
     * @return True if this parameter wasn't given an explicit value.
     */
    public boolean usingDefault()
    {
        return m_usingDefault;
    }

    
    public String describe() 
    {
        String message = getName();
        message += " - " + (isRequired() ? "REQUIRED" : "OPTIONAL");
        message += " - " + getDescription(); 
        return message;
    }
    
    /**
     * @return Returns the description.
     */
    public String getDescription()
    {
        return m_description;
    }

    /**
     * @return Returns the name.
     */
    public String getName()
    {
        return m_name;
    }

    /**
     * @return Returns the required.
     */
    public boolean isRequired()
    {
        return m_required;
    }

}
