/*
 * Created on Jan 16, 2004
 */
package com.mckessonaps.jspwiki.plugin.util;

import java.util.StringTokenizer;

/**
 * NOTE: This file is duplicated in the Visualizer project.  
 * Please keep in mind when bug-fixing. 
 * @author jvolkar
 */
public class WikiTextUtils
{
    private static final String SECTION_DIVIDER = "----";

    
    /**
     * Returns a section of wiki text from pageData.  Sections are delimited by
     * provided string, first section is number 1.
     * @param sectionNumber Must be greater than or equal to 1.
     * @param pageData must not be null
     * @param sectionDivider The delimiter of sections in the pageData.
     * @return May return an empty string if there aren't sectionNumber of sections in pageData.
     */
    public static String extractSectionData(int sectionNumber, String pageData, String sectionDivider)
    {
        int startBreakIndex = -1; //-1 to start since scanning loop below adds 1.
        if (sectionNumber > 1)
        {
            int count = 1;
            while (count < sectionNumber)
            {
                count++;
                startBreakIndex = pageData.indexOf(sectionDivider, startBreakIndex + 1);

                //If we could not find the next break, substitue an empty message
                if (-1 == startBreakIndex)
                {
                    pageData = "";
                    startBreakIndex = 0;
                    break;
                }
            }

            //Unless we are going to return the first section, we must skip over the break...
            if (startBreakIndex >= 0)
                startBreakIndex += sectionDivider.length();
        }

        //Assure that we don't still have a -1 as our start...
        startBreakIndex = Math.max(startBreakIndex, 0);

        int stopBreakIndex = pageData.indexOf(sectionDivider, startBreakIndex);

        //If we could not find the stop break, return all the rest of the page...
        if (stopBreakIndex <= startBreakIndex)
            return pageData.substring(startBreakIndex);
        else
            return pageData.substring(startBreakIndex, stopBreakIndex);
    }

    
    
    public static String wordWrap(String text, int length)
    {
        StringBuffer buffer = new StringBuffer( text.length() + 100 );
        StringTokenizer tokenizer = new StringTokenizer(text, "\n");
        while (tokenizer.hasMoreTokens() )
        {
            String line = tokenizer.nextToken();
            if (line.length() > length)
                wordWrap(buffer, line, length);
            else
                buffer.append( line );
        }
        
        return buffer.toString();
    }

    private static void wordWrap(StringBuffer buffer, String text, int length)
    {
        StringTokenizer tokenizer = new StringTokenizer( text, " ", true);
        int cumulativeLength = 0;
        while (tokenizer.hasMoreTokens())
        {
            String fragment = tokenizer.nextToken();
            
            if (fragment.equals(" ") && (0==cumulativeLength))
                continue; //Don't put spaces at beginning of lines...
            
            buffer.append( fragment );
            cumulativeLength += fragment.length();
            if (cumulativeLength >= length)
            {
                buffer.append( "\n" );
                cumulativeLength = 0;
            }
        }
    }
    
    
}
